@@ -21,6 +21,7 @@ pgsql_available = False
 PgsqlIntegrityError = None
 PgsqlOperationalError = None
 PgsqlDatabaseError = None
+PgsqlProgrammingError = None
 try:
     import psycopg2
     pgsql_available = True
@@ -28,6 +29,7 @@ try:
     PgsqlIntegrityError = psycopg2.IntegrityError
     PgsqlOperationalError = psycopg2.OperationalError
     PgsqlDatabaseError = psycopg2.Error
+    PgsqlProgrammingError = psycopg2.ProgrammingError
 except Exception as ex:
     #logging.error("Couldn't import psycopg2 - maybe that's ok for now - but shim the exception types.")
     class PgsqlIntegrityError(BaseException):
@@ -36,6 +38,8 @@ except Exception as ex:
         pass
     class PgsqlDatabaseError(BaseException):
         pass
+    class PgsqlProgrammingError(BaseException):
+        pass
 
 mysql_available = False
 MysqlIntegrityError = None
@@ -114,6 +118,7 @@ class PostgresConnection(object):
             args += " password={0}".format(password)
         self._db = None
         self._db_args = args
+        self._autocommit = autocommit
         try:
             self._connect(autocommit=autocommit)
         except Exception:
@@ -153,7 +158,7 @@ class PostgresConnection(object):
     def _cursor(self):
         """ Reconnect if disconnected. Then return a cursor. """
         if self._db is None:
-            self._connect()
+            self._connect(self._autocommit)
         return self._db.cursor()
 
     def __del__(self):
@@ -175,7 +180,6 @@ class PostgresConnection(object):
     def _execute(self, cursor, query, parameters, **kwargs):
         do_commit = kwargs.get('do_commit', True)
         try:
-            # return cursor.execute(query, parameters)
             cursor.execute(query, parameters)
             if do_commit:
                 self.commit()
@@ -244,8 +248,11 @@ class PostgresConnection(object):
         try:
             self._execute(cursor, query, parameters, do_commit=do_commit)
             if cursor.rowcount > 0:
-                last_row_id = cursor.fetchone()[0]
-                return last_row_id
+                try:
+                    return cursor.fetchone()[0]
+                except PgsqlProgrammingError as ex:
+                    logging.error("No Results even though cursor says rowcount > 0. Probably left out '... RETURNING row_id'")
+                    return None
         finally:
             if do_commit:
                 cursor.close()
