@@ -6,9 +6,12 @@
 
 # Python Imports
 import datetime
+import json
 import logging
 import random
+import os
 import signal
+import socket
 import sys
 import threading
 import time
@@ -177,6 +180,74 @@ class HydraHead(HydraThread):
         return '%s %s %s %s' % (self.name, hydra_type['hydra_type_name'], ("%d" % work_cnt).rjust(8), msg)
 
 
+    def deploy_build(self, hydra_queue, hydra_type):
+        #self.logw(hydra_queue, "deploy_build HYDRA QUEUE")
+        # host-specific by putting hostname: key in the work_data JSON
+        work_data = json.loads(hydra_queue['work_data'])
+        self.logw(work_data, "WORK DATA")
+        if work_data.get('hostname') == socket.gethostname() and work_data.get('env') == aegis.config.get('env'):
+            # Environment Settings
+            self.logw(socket.gethostname(), "HOSTNAME MATCH")
+            self.logw(aegis.config.get('env'), "ENV MATCH")
+            self.build = aegis.model.Build.get_id(work_data['build_id'])
+            app_dir = os.path.join(options.deploy_dir, options.program_name)
+            build_dir = os.path.join(app_dir, self.build['version'])
+            live_symlink = os.path.join(app_dir, aegis.config.get('env'))
+            self.start_t = time.time()
+            self.output_tx = ''
+            self.username, stderr, exit_status = aegis.stdlib.shell('whoami')
+            self.host = socket.gethostname()
+            # What to restart
+            processes, stderr, exit_status = aegis.stdlib.shell("sudo /usr/bin/supervisorctl status")
+            self.logw(processes, 'processes')
+            processes = [process.split(' ')[0] for process in processes.splitlines() if process.split(':')[0].endswith('_'+aegis.config.get('env'))]
+            self.logw(processes, "PROCESSES")
+            self.logw(build_dir, "BUILD DIR")
+            self.logw(live_symlink, "LIVE SYMLINK")
+            # Remove and re-link
+            if self.deploy_exec("rm %s" % live_symlink, cwd=app_dir):
+                logging.warning("ERROR RM")
+                return True, 1
+            if self.deploy_exec("ln -s %s %s" % (build_dir, live_symlink), cwd=app_dir):
+                logging.warning("ERROR LN")
+                return True, 1
+            # Restart processes
+            for process in processes:
+                self.logw(process, "RESTART")
+                if self.deploy_exec("sudo /usr/bin/supervisorctl restart %s" % (process), cwd=app_dir):
+                    self.logw(process, "ERROR RESTARTING PROCESS")
+                    return True, 1
+
+            self.deploy_done_exec()
+        return True, 1
+
+
+    def deploy_exec(self, cmd, cwd, env=None):
+        self.output_tx += "\n%s@%s:%s> %s\n" % (self.username, self.host, cwd, cmd)
+        stdout, stderr, exit_status = aegis.stdlib.shell(cmd, cwd=cwd, env=env)
+        self.output_tx += stdout
+        self.output_tx += "\n%s" % stderr
+        self.build.set_deploy_output(self.output_tx)
+        self.build = aegis.model.Build.get_id(self.build['build_id'])
+        if exit_status:
+            self.deploy_done_exec(exit_status)
+        return exit_status
+
+    def deploy_done_exec(self, exit_status=0):
+        end_t = time.time()
+        exec_t = end_t - self.start_t
+        self.output_tx += "\n  [ Exit %s   (%4.2f sec) ]" % (exit_status, exec_t)
+        self.build.set_deploy_output(self.output_tx, exit_status)
+
+    def revert_build(self, hydra_queue, hydra_type):
+        self.logw(hydra_queue, "revert_build HYDRA QUEUE")
+        self.logw(hydra_type, "revert_build HYDRA TYPE")
+        # Do a supervisor stop, then start
+        # host-specific by putting hostname: key in the work_data JSON
+        # capture output onto build deploy_output
+        # This is mostly calling the deploy_build() tho
+
+
 class Hydra(HydraThread):
 
     def __init__(self):
