@@ -501,3 +501,59 @@ class ReportType(aegis.database.Row):
     def set_sql(report_type_id, report_type_sql):
         sql = 'UPDATE report_type SET report_sql=%s WHERE report_type_id=%s'
         return db().execute(sql, report_type_sql, report_type_id)
+
+
+class Build(aegis.database.Row):
+    table_name = 'build'
+    id_column = 'build_id'
+
+    @classmethod
+    def scan(cls):
+        sql = "SELECT * FROM build ORDER BY build_id DESC"
+        return db().query(sql, cls=cls)
+
+    def set_build_output(self, output_tx, exit_status=None):
+        sql = "UPDATE build SET build_output_tx=%s, build_exit_status=%s WHERE build_id=%s"
+        return db().execute(sql, output_tx, exit_status, self['build_id'])
+
+    def set_deploy_output(self, output_tx, exit_status=None):
+        sql = "UPDATE build SET deploy_output_tx=%s, deploy_exit_status=%s WHERE build_id=%s"
+        return db().execute(sql, output_tx, exit_status, self['build_id'])
+
+    def set_revert_output(self, output_tx, exit_status=None):
+        sql = "UPDATE build SET revert_output_tx=%s, revert_exit_status=%s WHERE build_id=%s"
+        return db().execute(sql, output_tx, exit_status, self['build_id'])
+
+    def set_version(self, version):
+        sql = "UPDATE build SET version=%s WHERE build_id=%s"
+        return db().execute(sql, version, self['build_id'])
+
+    def set_previous_version(self, previous_version):
+        sql = "UPDATE build SET previous_version=%s WHERE build_id=%s AND previous_version IS NULL"
+        return db().execute(sql, previous_version, self['build_id'])
+
+    def set_revision(self, revision):
+        sql = "UPDATE build SET revision=%s WHERE build_id=%s"
+        return db().execute(sql, revision, self['build_id'])
+
+    def set_build_size(self, build_size):
+        sql = "UPDATE build SET build_size=%s WHERE build_id=%s"
+        return db().execute(sql, build_size, self['build_id'])
+
+    def set_deployed(self):
+        sql = "UPDATE build SET deploy_dttm=NOW() WHERE build_id=%s"
+        return db().execute(sql, self['build_id'])
+
+    def set_reverted(self):
+        sql = "UPDATE build SET revert_dttm=NOW() WHERE build_id=%s"
+        return db().execute(sql, self['build_id'])
+
+    @classmethod
+    def get_live_build(cls):
+        sql = "SELECT * FROM build WHERE deploy_dttm IS NOT NULL AND deploy_exit_status=0 AND revert_dttm IS NULL ORDER BY deploy_dttm DESC LIMIT 1"
+        return db().get(sql, cls=cls)
+
+    @classmethod
+    def get_version(cls, version):
+        sql = "SELECT * FROM build WHERE version=%s"
+        return db().get(sql, version, cls=cls)
