@@ -4,12 +4,16 @@
 # Aegis is your shield to protect you on the Brave New Web
 
 # Python Imports
+import configparser
 import copy
 import datetime
 import json
 import logging
 import os
+import socket
 import sys
+import threadpool
+import time
 import traceback
 import urllib
 
@@ -738,6 +742,224 @@ class AegisReport(AegisWeb):
             return self.render_path("reports.html", **self.tmpl)
 
 
+class AegisBuildForm(AegisWeb):
+    @tornado.web.authenticated
+    def get(self, build_id=None, *args):
+        self.enforce_admin()
+        self.tmpl['errors'] = {}
+        build_id = aegis.stdlib.validate_int(build_id)
+        if build_id:
+            self.tmpl['build'] = aegis.model.Build.get_id(build_id)
+        else:
+            self.tmpl['build'] = {}
+        self.tmpl['build_step'] = self.request.args.get('build_step', 'build')
+        return self.render_path("build_form.html", **self.tmpl)
+
+    @tornado.web.authenticated
+    def post(self, build_id=None, *args):
+        self.enforce_admin()
+        # Validate Input
+        self.tmpl['errors'] = {}
+        self.tmpl['build'] = build = {}
+        build['branch'] = self.request.args.get('branch')
+        build['revision'] = self.request.args.get('revision')
+        if not build['branch']:
+            self.tmpl['errors']['branch'] = '** required (string)'
+        if not build['revision']:
+            build['revision'] = 'HEAD'
+        if self.tmpl['errors']:
+            return self.render_path("build_form.html", **self.tmpl)
+        # Create build row and run it on the threadpool
+        build_id = aegis.model.Build.insert_columns(**build)
+        build_row = aegis.model.Build.get_id(build_id)
+        self.build_git_venv_yarn(build_row)
+        self.redirect('/aegis/build')
+
+    @threadpool.in_thread_pool
+    def build_git_venv_yarn(self, build):
+        try:
+            self.build = build
+            self.start_t = time.time()
+            self.output_tx = ''
+            # Environment
+            self.src_dir = aegis.config.get('src_dir')
+            self.src_repo = os.path.join(self.src_dir, options.program_name)
+            self.username, stderr, exit_status = aegis.stdlib.shell('whoami', cwd=self.src_dir)
+            self.host = socket.gethostname()
+            refspec = build['branch']
+            if build['revision'] != 'HEAD':
+                refspec = build['revision']
+            # Make a local clone of the repository from origin so we don't have to clone entire repository every time
+            if not os.path.exists(self.src_repo):
+                if self.build_exec("git clone --progress git@%s %s" % (aegis.config.get('git_repo'), options.program_name), cwd=self.src_dir):
+                    return
+            # Fetch all the changes to repo and set the correct branch and revision
+            if self.build_exec("git fetch --all", cwd=self.src_repo):
+                return
+            # Reset in case there are version changes from previous builds
+            if self.build_exec("git reset --hard", cwd=self.src_repo):
+                return
+            if self.build_exec("git checkout %s" % (refspec), cwd=self.src_repo):
+                return
+            if self.build_exec("git pull --commit --ff", cwd=self.src_repo):
+                return
+            if build['revision'] == 'HEAD':
+                commit_hash, stderr, exit_status = aegis.stdlib.shell('git rev-parse HEAD', cwd=self.src_repo)
+                self.build.set_revision(commit_hash)
+            # Generate new version number before cloning the new version tag into build directory
+            self.new_version()
+            env = {"GIT_COMMITTER_NAME": options.git_committer_name, "GIT_COMMITTER_EMAIL": options.git_committer_email,
+                   "GIT_AUTHOR_NAME": options.git_committer_name, "GIT_AUTHOR_EMAIL": options.git_committer_email}
+            if self.build_exec("git commit -m 'version %s' %s" % (self.next_tag, ' '.join(self.version_files)), cwd=self.src_repo, env=env):
+                return
+            if self.build_exec("git tag %s" % self.next_tag, cwd=self.src_repo):
+                return
+            if self.build_exec("git push", cwd=self.src_repo):
+                return
+            if self.build_exec("git push --tags", cwd=self.src_repo):
+                return
+            self.build.set_version(self.next_tag)
+            # Clone a fresh build into directory named by version tag
+            app_dir = os.path.join(options.deploy_dir, options.program_name)
+            self.build_dir = os.path.join(app_dir, self.next_tag)
+            if os.path.exists(self.build_dir):
+                if self.build_exec("rm -rf %s" % build_dir, cwd=app_dir):
+                    return
+            if self.build_exec("git clone %s %s" % (self.src_repo, self.build_dir), cwd=app_dir):
+                return
+            # Set up virtualenv
+            if self.build_exec("virtualenv --python=/usr/bin/python3 --system-site-packages virtualenv", cwd=self.build_dir):
+                return
+            if self.build_exec("virtualenv/bin/pip --cache-dir .cache install -e .", cwd=self.build_dir):
+                return
+            # Set up and run yarn if it's installed
+            self.yarn, stderr, exit_status = aegis.stdlib.shell('which yarn', cwd=self.src_dir)
+            if self.yarn:
+                if self.build_exec("nice yarn install", cwd=self.build_dir):
+                    return
+                if self.build_exec("nice yarn run %s --cache-folder /srv/www/.cache/yarn" % options.env, cwd=self.build_dir):
+                    return
+                build_output_file = os.path.join(self.build_dir, options.build_output_file % {'tag': self.next_tag})
+                if os.path.exists(build_output_file):
+                    build_size = os.path.getsize(build_output_file)
+                    if build_size:
+                        self.build.set_build_size(build_size)
+            # Rsync the files to the servers if it's configured
+            rsync_hosts = [rh for rh in aegis.config.get('rsync_hosts') if rh != self.host]
+            for rsync_host in rsync_hosts:
+                cmd = "rsync -q --password-file=/etc/rsync.password -avzhW %s www-data@%s::%s" % (self.build_dir, rsync_host, options.rsync_module)
+                if self.build_exec(cmd, cwd=self.build_dir):
+                    return
+
+        except Exception as ex:
+            logging.exception(ex)
+            self.output_tx += "\n%s" % traceback.format_exc()
+            self.build.set_build_output(self.output_tx, 1)
+            return self.done_exec(1)
+
+        return self.done_exec()
+
+    def build_exec(self, cmd, cwd, env=None):
+        self.output_tx += "\n%s@%s:%s> %s\n" % (self.username, self.host, cwd, cmd)
+        stdout, stderr, exit_status = aegis.stdlib.shell(cmd, cwd=cwd, env=env)
+        self.output_tx += stdout
+        self.output_tx += "\n%s" % stderr
+        self.build.set_build_output(self.output_tx)
+        self.build = aegis.model.Build.get_id(self.build['build_id'])
+        if exit_status:
+            self.done_exec(exit_status)
+        return exit_status
+
+    def done_exec(self, exit_status=0):
+        end_t = time.time()
+        exec_t = end_t - self.start_t
+        self.output_tx += "\n  [ Exit %s   (%4.2f sec) ]" % (exit_status, exec_t)
+        self.build.set_build_output(self.output_tx, exit_status)
+
+    def new_version(self):
+        self.branch = self.build['branch']
+        self.section = '%s' % self.branch
+        self.config = configparser.ConfigParser()
+        self.version_file = os.path.join(self.src_repo, options.program_name, 'version.cfg')
+        self.config.read(self.version_file)
+        try:
+            self.version = self.str_version(self.config.get(self.section, 'version'))
+        except configparser.NoSectionError as ex:
+            self.config.add_section(self.section)
+            self.version = [0, 0, 0]
+        self.next_version = self.incr_version(*self.version)
+        self.tag = '%s-%s' % (self.branch, self.version_str(*self.version))
+        self.next_tag = '%s-%s' % (self.branch, self.version_str(*self.next_version))
+        self.write_py_version()
+        self.write_custom_version()
+
+    def incr_version(self, x, y, z):
+        if z < 99:
+            return x, y, z+1
+        z = 0
+        if y < 9:
+            return x, y+1, z
+        y = 0
+        return x+1, y, z
+
+    def str_version(self, cfg_version):
+        version_number = cfg_version.split('-')[-1]
+        return tuple([int(subversion) for subversion in version_number.split('.')])
+
+    def version_str(self, x, y, z):
+        return '%s.%s.%s' % (x, y, z)
+
+    def write_py_version(self):
+        self.config.set(self.section, 'version', self.next_tag)
+        fd = open(self.version_file, 'w')
+        self.config.write(fd)
+        fd.close()
+        self.version_files = [self.version_file]
+
+    def write_custom_version(self):
+        pass
+
+
+class AegisBuild(AegisWeb):
+    @tornado.web.authenticated
+    def get(self, *args):
+        self.enforce_admin()
+        self.tmpl['builds'] = aegis.model.Build.scan()
+        self.tmpl['live_build'] = aegis.model.Build.get_live_build()
+        return self.render_path("build.html", **self.tmpl)
+
+    @tornado.web.authenticated
+    def post(self, *args):
+        self.enforce_admin()
+        build_id = [aegis.stdlib.validate_int(k.replace('deploy_', '')) for k in self.request.args.keys() if k.startswith('deploy_')][0]
+        if build_id:
+            hydra_type = aegis.model.HydraType.get_name('deploy_build')
+            # Put an item on the work queue to signal each host to deploy
+            for deploy_host in options.deploy_hosts:
+                hydra_queue = {'hydra_type_id': hydra_type['hydra_type_id'], 'priority_ndx': hydra_type['priority_ndx'], 'work_dttm': aegis.database.Literal("NOW()")}
+                work_data = {'hostname': deploy_host, 'env': aegis.config.get('env'), 'build_id': build_id}
+                hydra_queue['work_data'] = json.dumps(work_data, cls=aegis.stdlib.DateTimeEncoder)
+                hydra_queue_id = aegis.model.HydraQueue.insert_columns(**hydra_queue)
+            # Set the previous version so we know what to revert back to, and mark it deployed
+            build = aegis.model.Build.get_id(build_id)
+            live_build = aegis.model.Build.get_live_build()
+            if live_build:
+                build.set_previous_version(build['version'])
+                build = aegis.model.Build.get_id(build['build_id'])
+            build.set_deployed()
+            return self.redirect('/aegis/build')
+        build_id = [aegis.stdlib.validate_int(k.replace('revert_', '')) for k in self.request.args.keys() if k.startswith('revert_')][0]
+        if build_id:
+            hydra_type = aegis.model.HydraType.get_name('revert_build')
+            # Put an item on the work queue to signal each host to deploy
+            for deploy_host in options.deploy_hosts:
+                hydra_queue = {'hydra_type_id': hydra_type['hydra_type_id'], 'priority_ndx': hydra_type['priority_ndx'], 'work_dttm': aegis.database.Literal("NOW()")}
+                work_data = {'hostname': deploy_host, 'env': aegis.config.get('env'), 'build_id': build_id}
+                hydra_queue['work_data'] = json.dumps(work_data, cls=aegis.stdlib.DateTimeEncoder)
+                hydra_queue_id = aegis.model.HydraQueue.insert_columns(**hydra_queue)
+            return self.redirect('/aegis/build')
+
+
 class AegisHome(AegisWeb):
     @tornado.web.authenticated    # Could do something like @aegis.webapp.admin_only which also sends for login but then rejects if not admin
     def get(self, *args):
@@ -746,6 +968,9 @@ class AegisHome(AegisWeb):
 
 
 handler_urls = [
+    (r'^/aegis/build/add\W*$', AegisBuildForm),
+    (r'^/aegis/build/(\d+)\W*$', AegisBuildForm),
+    (r'^/aegis/build\W*$', AegisBuild),
     (r'^/aegis/hydra/add\W*$', AegisHydraForm),
     (r'^/aegis/hydra/(\d+)\W*$', AegisHydraForm),
     (r'^/aegis/hydra\W*$', AegisHydra),
